@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-__version__ = '2.1.0'
+__version__ = '2.2.0'
 __versionfull__ = __version__
 
 import functools
@@ -7,11 +7,13 @@ import hashlib
 import inspect
 import logging
 import uuid
+import sys
 
 from django.conf import settings
 from django.core.cache import cache as default_cache
 from django.core.cache.backends.base import DEFAULT_TIMEOUT
 from django.utils.encoding import force_bytes
+from django.utils.six import text_type, string_types, iteritems
 
 logger = logging.getLogger(__name__)
 
@@ -24,12 +26,16 @@ DEFAULT_CACHE_OBJECT = DefaultCacheObject()
 
 
 def _get_argspec(f):
-    try:
-        argspec = inspect.getargspec(f)
-    except ValueError:
-        # this can happen in python 3.5 when
-        # function has keyword-only arguments or annotations
+    if sys.version_info[:2] >= (3, 0):
+        # inspect has deprecated getargspec since version 3.0
         argspec = inspect.getfullargspec(f)
+    else:
+        try:
+            argspec = inspect.getargspec(f)
+        except ValueError:
+            # this can happen in python 3.5 when
+            # function has keyword-only arguments or annotations
+            argspec = inspect.getfullargspec(f)
     return argspec
 
 
@@ -86,6 +92,7 @@ class Memoizer(object):
     """
     This class is used to control the memoizer objects.
     """
+
     def __init__(self, cache=default_cache, cache_prefix='memoize',
                  default_cache_value=DEFAULT_CACHE_OBJECT):
         self.cache = cache
@@ -133,7 +140,9 @@ class Memoizer(object):
         self.cache.set_many(data=mapping, timeout=timeout)
 
     def _memvname(self, funcname):
-        return funcname + '_memver'
+        return hashlib.md5(
+            force_bytes(funcname)
+        ).hexdigest() + "_memver"
 
     def _memoize_make_version_hash(self):
         return uuid.uuid4().hex
@@ -182,7 +191,12 @@ class Memoizer(object):
 
         return fname, ''.join(version_data_list)
 
-    def _memoize_make_cache_key(self, make_name=None, timeout=DEFAULT_TIMEOUT):
+    def _memoize_make_cache_key(
+            self,
+            make_name=None,
+            timeout=DEFAULT_TIMEOUT,
+            convert_to_unicode=False
+    ):
         """
         Function used to create the cache_key for memoized functions.
         """
@@ -205,6 +219,41 @@ class Memoizer(object):
             else:
                 keyargs, keykwargs = args, kwargs
 
+            # convert string-type parameters to `unicode`, if necessary
+            if convert_to_unicode:
+                # convert function name to `unicode`
+                if isinstance(altfname, string_types):
+                    altfname = text_type(altfname)
+
+                keyargs = list(keyargs)
+
+                # convert all string values in `args` to `unicode`
+                for i, elem in enumerate(keyargs):
+                    if isinstance(elem, string_types):
+                        keyargs[i] = text_type(elem)
+
+                keyargs = tuple(keyargs)
+
+                # we have to save the keys here as the dictionary may mutate
+                # during the course of iteration
+                keys = list(keykwargs.keys())
+
+                # convert all string keys in `kwargs` to `unicode`
+                for k in keys:
+                    value = keykwargs[k]
+
+                    if isinstance(k, string_types):
+                        new_k = text_type(k)
+                        keykwargs[new_k] = value
+                        # avoid doubling of keys
+                        if type(new_k) != type(k):
+                            del keykwargs[k]
+
+                # convert all string values in `kwargs` to `unicode`
+                for k, v in iteritems(keykwargs):
+                    if isinstance(v, string_types):
+                        keykwargs[k] = text_type(v)
+
             cache_key = hashlib.md5(
                 force_bytes((altfname, keyargs, keykwargs))
             ).hexdigest()
@@ -274,7 +323,13 @@ class Memoizer(object):
 
         return tuple(new_args), kwargs
 
-    def memoize(self, timeout=DEFAULT_TIMEOUT, make_name=None, unless=None):
+    def memoize(
+            self,
+            timeout=DEFAULT_TIMEOUT,
+            make_name=None,
+            unless=None,
+            convert_to_unicode=False
+    ):
         """
         Use this to cache the result of a function, taking its arguments into
         account in the cache key.
@@ -317,17 +372,25 @@ class Memoizer(object):
 
                     readable and writable
 
+                **convert_to_unicode**
+                    A boolean flag to enable conversion of strings to `unicode`
+                    objects. Primarily used when generating the cache_key.
+
+                    readable and writable
+
 
         :param timeout: Default: 300. If set to an integer, will cache
                         for that amount of time. Unit of time is in seconds.
-        :param make_name: Default None. If set this is a function that accepts
+        :param make_name: Default: None. If set this is a function that accepts
                           a single argument, the function name, and returns a
                           new string to be used as the function name.
                           If not set then the function name is used.
-        :param unless: Default None. Cache will *always* execute the caching
-                       facilities unelss this callable is true.
+        :param unless: Default: None. Cache will *always* execute the caching
+                       facilities unless this callable is true.
                        This will bypass the caching entirely.
-
+        :param convert_to_unicode: Default: False. If set, will determine if
+                                   string objects are converted to `unicode`
+                                   during the cache_key generation process.
         """
 
         def memoize(f):
@@ -337,6 +400,7 @@ class Memoizer(object):
                 if callable(unless) and unless() is True:
                     return f(*args, **kwargs)
 
+                # try to fetch the function's return value from the cache
                 try:
                     cache_key = decorated_function.make_cache_key(
                         f, *args, **kwargs
@@ -350,6 +414,8 @@ class Memoizer(object):
                     )
                     return f(*args, **kwargs)
 
+                # if a cache miss occurs, run the function from scratch
+                # and cache the resulting return value
                 if rv == self.default_cache_value:
                     rv = f(*args, **kwargs)
                     try:
@@ -368,7 +434,7 @@ class Memoizer(object):
             decorated_function.uncached = f
             decorated_function.cache_timeout = timeout
             decorated_function.make_cache_key = self._memoize_make_cache_key(
-                make_name, decorated_function
+                make_name, decorated_function, convert_to_unicode
             )
             decorated_function.delete_memoized = (
                 lambda: self.delete_memoized(f)
